/*
 * vfwrite.c version 2.0 part of the vfloppy 2.0 package
 *
 * This version delivered in 2010 by Fred Jan Kraan (fjkraan@xs4all.nl)
 *
 * vfwrite is placed under the GNU General Public License in March 2010.
 *
 *  This file is part of Vfloppy 2.0.
 *
 *  Vfloppy is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Vfloppy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Vfloppy; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 * vfwrite - writes a file to the image imageName
 * 
 * vfwrite <imageName> <fileName>
 */

#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <fcntl.h>
#include "vf.h"

int debug = 0;

int main(int argc,char **argv)
{	
	argData_t *pArgData = &argData;
	parseArgs(argc, argv, pArgData);
	debug = pArgData->debug;
	
	if (pArgData->imageName == NULL || pArgData->fileName == NULL)
	{
		fprintf(stderr, "%s <imageName> <fileName>\n", argv[0]);
		fprintf(stderr, "vferase 2.1 (c)2010 F.J. Kraan\n");
		exit(1);
	}

	if (checkFileExistance(pArgData->imageName) == 0)
	{
		printf("Image '%s' not found, exiting.\n", argv[1]);
	}
	char *imageName = pArgData->imageName;
	FILE *imageFile = NULL;
	
	fileData_t *pFileData = &fileData;

	pFileData->fileName = (unsigned char*)pArgData->fileName;
	
	imageFile = openFileRW(imageName);	// open image

	convertFileName(pFileData);
	
	findFileExtends(imageFile, pFileData);
	
	if (pFileData->found == 1) 
	{
		if (debug > 1) printf("main; imageFile: %p, pFileData: %p\n", imageFile, pFileData);
		eraseFile(imageFile, pFileData);
	}
	fclose(imageFile);		// close image
	return 0;
}
